/*
    libvice - SGI O2 Video Image Compression Engine Library
    Copyright (C) 2002 Vivien Chappelier
                       Ilya Volynets

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "vice.h"

#undef DEBUG

int vice_msp_start(vice_t* vice, msp_run* run)
{
    vice_write_reg32(vice, MSP_PC, MSP_IRAM);
#ifdef DEBUG
    fprintf(stderr, "MSP_CAUSE=%08x, MSP_PC=%08x\n",
            vice_read_reg32(&vice,MSP_CAUSE),
            vice_read_reg32(&vice,MSP_PC));
#endif

    if(ioctl(vice->fd,VICE_IOCTL_MSP_RUN,run) == -1) {
	fprintf(stderr, "error running MSP: %s\n", strerror(errno));
	return 0;
    }
    return 1;
}

int vice_msp_load_code(vice_t *vice, void *code, int size)
{
    int i;

    if(size >= 0x3000) {
      fprintf(stderr, "code doesn't fit in VICE MSP instruction RAM\n");
      return(0);
    }
    for(i = 0; i < size; i += 4)
      vice_write_reg32(vice, MSP_IRAM + i, ((u32*) code)[i>>2]);
    return(1);
}

void vice_debug_msp_dump_iram(vice_t *vice)
{
    u32 i = MSP_IRAM;
    while(i < 0x3000) {
	u64 insns;
	insns = vice_read_reg(vice, i);
	if((i % 32) == 0)
	    printf("\n%x: ", i);
	fprintf(stderr, "%08x %08x ",((u32*) &insns)[0], ((u32*) &insns)[1]);
	i += 8;
    }
}
