

#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "vice.h"

#define HI_16(x) (((x)>>16)&0xFFFF)
#define LO_16(x) ((x)&0xFFFF)

int read_msp_code(vice_t *vice, u32 *msp_code, int length)
{
    int i = 0;
    int msp_code_fd = open("msp_code",O_RDONLY);
    int msp_code_len;

    if(msp_code_fd == -1) {
	printf ("Error openning msp code: %s\n",strerror(errno));
	return 0;
    }

    msp_code_len = read(msp_code_fd, msp_code, length);
    close (msp_code_fd);
    return msp_code_len;
}

void pgm_write_header(FILE *file, int width, int height)
{
  fprintf(file, "P5 %ld %ld 255\n", width, height);
}

void pgm_read_header(FILE *file, int *width, int *height)
{
  char comment[70];
  int byte;
  int value[3] = {0,0,0}, count =0;

  /* check we are dealing with a byte pgm file (Ascii pgm not supported) */
  if(fgetc(file) != 'P' || fgetc(file) != '5')
  {
    fprintf(stderr,"Error in pgm_read_image_header_bytes(): file is not a byte format pgm file!");
    return;
  }

  while(count < 3)
  {
      /* read bytes, check for comments and valid numbers */
      switch (byte = getc(file))
      {
	case '#' : fgets(comment, 70, file); break; 
	case ' ' : break;
	case '\n': break;
	case '\r': break;
        default  : ungetc(byte, file);
                   fscanf(file, "%d", &value[count]); 
		   count++;
		   break;
      }
  }

  fgets(comment, 70, file); /* skip until end of line */

  *width = value[0];
  *height = value[1];

  return;
}

extern char msp_code_start;
extern char msp_code_end;

int main(int *argc, int *argv)
{
    vice_t *vice;
    volatile vice_dma_desc* dma;
    dma_run run;
    msp_run mrun={0,0,0};
    int i=0;
    unsigned long long ul;
    FILE *file_in, *file_out;
    int width, height;
    //    u32 msp_code[512];

    fprintf(stderr, "Loading pgm\n");
    file_in = fopen("test.pgm", "rb");
    pgm_read_header(file_in, &width, &height);

    /* start VICE */
    fprintf(stderr, "Starting VICE\n");
    if(!(vice = vice_init(width*height * 3, 1)))
	return 1;

    /* load our code */
    fprintf(stderr, "load MSP code (%d bytes)\n", &msp_code_end-&msp_code_start);
    //    i = read_msp_code(vice, msp_code, 512 * 4);
    //    fprintf(stderr, "load MSP code (%d bytes)\n", i);
    if(!vice_msp_load_code(vice, &msp_code_start, &msp_code_end-&msp_code_start))
      return 1;

    /* cleanup data ram */
    fprintf(stderr, "fill MSP ram\n");
    vice_fill_dram(vice, 0);

    /* read pgm in VICE I/O buffer */
    fprintf(stderr, "reading file\n");
    fread(vice->buffer, width*height, 2, file_in);
    fclose(file_in);

    /* run MSP program */
    fprintf(stderr, "Trying MSP\n");
    vice_msp_start(vice, &mrun);
    printf("result = %x\n", mrun.reason);

    /* write processed image */
    fprintf(stderr, "Writing processed pgm\n");
    file_out = fopen("processed.pgm", "wb");
    pgm_write_header(file_out, width, height);
    fwrite(vice->buffer, width*height, 2, file_out);
    fclose(file_out);

    vice_cleanup(vice);
}

