#ifndef VICE_H
#define VICE_H

#include <linux/vice.h>

typedef struct vice_t
{
    int fd;
    void *regs;
    void *buffer;
    int size;
} vice_t;

#define u64 unsigned long long
#define u32 unsigned int

static inline void vice_write_reg(vice_t *v, u32 reg, u64 value)
{
    *((u64*) (v->regs + reg)) = value;
}

static inline u64 vice_read_reg(vice_t *v, u32 reg)
{
    return *((u64*) (v->regs + reg));
}

static inline void vice_write_reg32(vice_t *v, u32 reg, u32 value)
{
    *((u32*) (v->regs + reg)) = value;
}

static inline u32 vice_read_reg32(vice_t *v, u32 reg)
{
    return *((u32*) (v->regs + reg));
}

vice_t *vice_init(int size, int verbose);
void vice_cleanup(vice_t *vice);
void vice_debug_dump_dram(vice_t *vice);
void vice_fill_dram(vice_t *vice, u32 value);
int vice_load_data(vice_t *vice, int offset, void *data, int size);
int vice_msp_start(vice_t* vice, msp_run* run);
int vice_msp_load_code(vice_t *vice, void *code, int size);
void vice_debug_msp_dump_iram(vice_t *vice);
volatile vice_dma_desc *vice_dma_get_desc(vice_t *vice, int chnum, int dnum);
int vice_dma_start(vice_t *vice, dma_run *run);

#endif
